@@ -26,7 +26,7 @@
 
 import logging, os, os.path, signal, sys, re
 
-from gi.repository import GObject
+from gi.repository import GLib
 import dbus
 import dbus.service
 import dbus.mainloop.glib
@@ -133,14 +133,14 @@ class Backend(dbus.service.Object):
         once the server is ready to take requests.
         '''
         dbus.service.Object.__init__(self, self.bus, '/RecoveryMedia')
-        self.main_loop = GObject.MainLoop()
+        self.main_loop = GLib.MainLoop()
         self._timeout = False
         if timeout:
             def _quit():
                 """This function is ran at the end of timeout"""
                 self.main_loop.quit()
                 return True
-            GObject.timeout_add(timeout * 1000, _quit)
+            GLib.timeout_add(timeout * 1000, _quit)
 
         # send parent process a signal that we are ready now
         if send_usr1:
@@ -238,7 +238,7 @@ class Backend(dbus.service.Object):
     # Internal API for calling from Handlers (not exported through D-BUS)
     #
 
-    def request_mount(self, recovery, sender=None, conn=None):
+    def request_mount(self, recovery, type="r", sender=None, conn=None):
         '''Attempts to mount the recovery partition
 
            If successful, return mntdir.
@@ -262,7 +262,7 @@ class Backend(dbus.service.Object):
 
         #if not already, mounted, produce a mount point
         mntdir = tempfile.mkdtemp()
-        mnt_args = ['mount', '-r', recovery, mntdir]
+        mnt_args = ['mount', '-%s' %type, recovery, mntdir]
         if ".iso" in recovery:
             mnt_args.insert(1, 'loop')
             mnt_args.insert(1, '-o')
@@ -458,7 +458,7 @@ class Backend(dbus.service.Object):
 
         self._reset_timeout()
 
-        base_mnt = self.request_mount(base, sender, conn)
+        base_mnt = self.request_mount(base, "r", sender, conn)
 
         assembly_tmp = tempfile.mkdtemp()
         atexit.register(walk_cleanup, assembly_tmp)
@@ -571,7 +571,7 @@ class Backend(dbus.service.Object):
             if err:
                 logging.debug("error during isoinfo invokation: %s", err)
         else:
-            mntdir = self.request_mount(iso, sender, conn)
+            mntdir = self.request_mount(iso, "r", sender, conn)
 
             if os.path.exists(os.path.join(mntdir, '.disk', 'info')):
                 with open(os.path.join(mntdir, '.disk', 'info'), 'r') as rfd:
@@ -653,7 +653,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                     version = test_initrd(['isoinfo', '-J', '-i', recovery, '-x', '/casper/initrd.lz'])
 
         else:
-            mntdir = self.request_mount(recovery, sender, conn)
+            mntdir = self.request_mount(recovery, "r", sender, conn)
             if os.path.exists(os.path.join(mntdir, 'bto.xml')):
                 self.xml_obj.load_bto_xml(os.path.join(mntdir, 'bto.xml'))
                 version = self.xml_obj.fetch_node_contents('iso')
@@ -811,13 +811,23 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         #emulate fat32 partition automatically setting sh and py scripts executable
         for root, dirs, files in os.walk(self.package_dir, topdown=False):
             for name in files:
+                if name.endswith('.sh'):
+                    with open(os.path.join(root, name), 'r') as rfd:
+                        script = rfd.readlines()
+                    with open(os.path.join(root, name), 'w') as wfd:
+                        for line in script:
+                            if '/cdrom' in line:
+                                line = line.replace('/cdrom', self.package_dir)
+                            elif 'apt-get purge' in line:
+                                line = line.replace('\n', ' || true\n')
+                            wfd.write(line)
                 if name.endswith('.sh') or name.endswith('.py'):
                     os.chmod(os.path.join(root, name), 0o755)
 
         #mount RP (if possible)
         if recovery:
             self.report_progress("Mounting recovery partition.")
-            mntdir = self.request_mount(recovery)
+            mntdir = self.request_mount(recovery, "r")
             #build the pool with stuff from RP
             for directory in ['debs', 'pool']:
                 if os.path.exists(os.path.join(mntdir, directory)):
@@ -836,7 +846,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                              stdout=subprocess.PIPE, 
                              cwd=debs_dir)
             output = build.communicate()[0]
-            with open(os.path.join(debs_dir, 'Packages'), 'w') as wfd:
+            with open(os.path.join(debs_dir, 'Packages'), 'wb') as wfd:
                 wfd.write(output)
 
             source_list = os.path.join('/etc', 'apt', 'sources.list.d', os.path.basename(self.package_dir) + '.list')
@@ -910,16 +920,25 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         environment = os.environ
         environment['DEBIAN_FRONTEND'] = 'noninteractive'
         environment['PATH'] = '/sbin:/usr/sbin:/bin:/usr/bin'
+        scripts.append('/usr/bin/apt-get -f install --yes')
+        scripts.append('/usr/bin/ubuntu-drivers autoinstall')
         num = float(1)
         total = len(scripts)
         for script in scripts:
+            if 'installer' in script:
+                logging.debug('skipping script: %s' % script)
+                continue
             logging.debug('executing script: %s' % script)
             self.report_progress("executing %s" % script, num/total)
             #in case shell script is missing starting shebang (a few are)
             if script.endswith('.sh'):
                 cmd = ['/bin/sh', script]
             else:
-                cmd = [script]
+                #for apt-get and ubuntu-drivers
+                if ' ' in script:
+                    cmd = script.split()
+                else:
+                    cmd = [script]
             logging.debug(fetch_output(cmd, None, environment))
             num+=1
         logging.debug("leave _process_scripts")
@@ -933,16 +952,46 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         self._reset_timeout()
         self._check_polkit_privilege(sender, conn, 'com.dell.recoverymedia.restore')
         logging.debug("enable_boot_to_restore")
+        self._prepare_reboot("99_dell_recovery")
 
+    @dbus.service.method(DBUS_INTERFACE_NAME,
+        in_signature = 'ss', out_signature = '', sender_keyword = 'sender',
+        connection_keyword = 'conn')
+    def enable_boot_to_utility(self, utility, executable, sender=None, conn=None):
+        """Enables the default one-time boot option to be utility partition"""
+        self._reset_timeout()
+        self._check_polkit_privilege(sender, conn, 'com.dell.recoverymedia.utility')
+        #prep the content for UP
+        mntdir = self.request_mount(utility, "rw")
+        target = os.path.join(mntdir, os.path.basename(executable))
+        if os.path.exists(target):
+            os.remove(target)
+        shutil.copy(executable, target)
+        old = False
+        target = os.path.join(mntdir, 'autoexec.bat')
+        if os.path.exists(target):
+            shutil.move(target, 
+                        os.path.join(mntdir, 'autoexec.old'))
+            old = True
+        with open (target, 'w') as wfd:
+            wfd.write("DEL AUTOEXEC.BAT\r\n")
+            if old:
+                wfd.write("REN AUTOEXEC.OLD AUTOEXEC.BAT\r\n")
+            wfd.write("%s\r\n" % os.path.basename(executable))
+        
+        self._prepare_reboot("98_dell_bios")
+
+    def _prepare_reboot(self, dest):
+        """Helper function to reboot into an entry"""
         #find our one time boot entry
-        if not os.path.exists("/etc/grub.d/99_dell_recovery"):
-            raise RestoreFailed("missing 99_dell_recovery to parse")
+        if not os.path.exists("/etc/grub.d/%s" % dest):
+            raise RestoreFailed("missing %s to parse" % dest) 
 
-        with open('/etc/grub.d/99_dell_recovery') as rfd:
-            dell_rec_file = rfd.readlines()
+        with open('/etc/grub.d/%s' % dest) as rfd:
+            grub_file = rfd.readlines()
 
         entry = False
-        for line in dell_rec_file:
+        for line in grub_file:
             if "menuentry" in line:
                 split = line.split('"')
                 if len(split) > 1:
@@ -950,7 +999,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                     break
 
         if not entry:
-            raise RestoreFailed("Error parsing 99_dell_recovery for bootentry.")
+            raise RestoreFailed("Error parsing %s for bootentry." % dest)
 
         #set us up to boot saved entries
         with open('/etc/default/grub', 'r') as rfd:
@@ -995,7 +1044,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
         atexit.register(walk_cleanup, tmpdir)
 
         #mount the recovery partition
-        mntdir = self.request_mount(recovery, sender, conn)
+        mntdir = self.request_mount(recovery, "r", sender, conn)
 
         #validate that ubuntu is on the partition
         if not os.path.exists(os.path.join(mntdir, '.disk', 'info')):
@@ -1019,7 +1068,7 @@ arch %s, distributor_str %s" % (bto_version, distributor, release, arch, distrib
                                            w_size)
             black_tree("copy", pattern, mntdir, tmpdir)
             self.stop_progress_thread()
-            mntdir = self.request_mount(os.path.join(mntdir, 'ubuntu.iso'), sender, conn)
+            mntdir = self.request_mount(os.path.join(mntdir, 'ubuntu.iso'), "r", sender, conn)
 
         #Generate BTO XML File
         self.xml_obj.replace_node_contents('date', str(datetime.date.today()))
